unit PM_XML;

interface

Uses  SysUtils, Variants, Classes, PM_CommData, PM_DataModule, PM_CommFunction;

function CreateNewPMFile: boolean;
function WriteSettingsToXML: boolean;
function ReadSettingsFromXML: boolean;
function WriteHostToXml(h:THostRecord;n:boolean):boolean;
function HostCount:integer;
function ReadHostFromXml(node:string):THostRecord;
function ChildNodeName(i:integer): string;
function DeleteXmlHost(n: string):boolean;
function LoadProtocols: boolean;
function LoadTypes: boolean;


implementation

function CreateNewPMFile: boolean;
var
  NewFile: TStringList;
begin
  result:=True;
  NewFile:=TStringList.Create;
  NewFile.Append('<?xml version="1.0" encoding="UTF-8"?>');
  NewFile.Append('<PM>');
  NewFile.Append('  <Settings>');
  NewFile.Append('    <autosave />');
  NewFile.Append('  </Settings>');
  NewFile.Append('  <Protocols>');
  NewFile.Append('  </Protocols>');
  NewFile.Append('  <Types>');
  NewFile.Append('  </Types>');
  NewFile.Append('  <Hosts>');
  NewFile.Append('  </Hosts>');
  NewFile.Append('</PM>');
  try
    NewFile.SaveToFile('PM.xml');
  except
    result:=False;
  end;
  NewFile.Free;
end;

function WriteSettingsToXML: boolean;
begin
  result:=True;
  try
    with DataModule1.XML1.DocumentElement.ChildNodes['Settings'].ChildNodes['autosave'] do
    begin
      SetAttributeNS('width', '',IntToStr(AppSettings.Width));
      SetAttributeNS('height', '',IntToStr(AppSettings.Height));
      SetAttributeNS('left', '',IntToStr(AppSettings.Left));
      SetAttributeNS('top', '',IntToStr(AppSettings.Top));
      SetAttributeNS('dleft', '',IntToStr(AppSettings.dLeft));
      SetAttributeNS('dtop', '',IntToStr(AppSettings.dTop));
      SetAttributeNS('fontsize', '',IntToStr(AppSettings.FontSize));
      //SetAttributeNS('autohide', '',BoolToStr(AppSettings.Autohide));
      SetAttributeNS('ontop', '',BoolToStr(AppSettings.OnTop));
      SetAttributeNS('visiblecols', '',BArrayToStr(AppSettings.VisibleCols));
      SetAttributeNS('sorted', '',IntToStr(AppSettings.Sorted));
      SetAttributeNS('sortkindasc', '',BoolToStr(AppSettings.SortKindAsc));
      SetAttributeNS('pmrow', '',AppSettings.PMRow);
    end;
  except
    result:=False;
  end;
end;

function ReadSettingsFromXML: boolean;
begin
  result:=True;
  try
    with DataModule1.XML1.DocumentElement.ChildNodes['Settings'].ChildNodes['autosave'] do
    begin
      AppSettings.Width:=StrToInt(Attributes['width']);
      AppSettings.Height:=StrToInt(Attributes['height']);
      AppSettings.Left:=StrToInt(Attributes['left']);
      AppSettings.Top:=StrToInt(Attributes['top']);
      AppSettings.dLeft:=StrToInt(Attributes['dleft']);
      AppSettings.dTop:=StrToInt(Attributes['dtop']);
      AppSettings.FontSize:=StrToInt(Attributes['fontsize']);
      //AppSettings.Autohide:=StrToBool(Attributes['autohide']);
      AppSettings.OnTop:=StrToBool(Attributes['ontop']);
      AppSettings.VisibleCols:=StrToBArray(Attributes['visiblecols']);
      AppSettings.Sorted:=StrToInt(Attributes['sorted']);
      AppSettings.SortKindAsc:=StrToBool(Attributes['sortkindasc']);
      AppSettings.PMRow:=Attributes['pmrow'];
    end;
  except
    result:=False;
  end;
end;

function WriteHostToXml(h:THostRecord;n:boolean):boolean;
begin
  result:=True;
  try
    if n then DataModule1.XML1.DocumentElement.ChildNodes['Hosts'].AddChild(h.id);
    with DataModule1.XML1.DocumentElement.ChildNodes['Hosts'].ChildNodes[h.id] do
    begin
      SetAttributeNS('host', '',h.Host);
      SetAttributeNS('ip', '',  h.IP);
      SetAttributeNS('user', '',h.User);
      SetAttributeNS('prot', '',h.Protocol);
      SetAttributeNS('port', '',h.Port);
      SetAttributeNS('path', '',h.Path);
      SetAttributeNS('verb', '',h.Verbosity);
      SetAttributeNS('sess', '',h.Session);
      SetAttributeNS('type', '',h.HostType);
      SetAttributeNS('desc', '',h.Description);
    end;
    DataModule1.XML1.Active:=False;
    DataModule1.XML1.Active:=True;
  except
    result:=False;
  end;
end;

function ReadHostFromXml(node:string):THostRecord;
begin
  try
    with DataModule1.XML1.DocumentElement.ChildNodes['Hosts'].ChildNodes[node] do
    begin
      result.id:=LocalName;
      result.Host:=Attributes['host'];
      result.IP:=Attributes['ip'];
      result.HostType:=StrToIntDef(Attributes['type'],0);
      result.Protocol:=Attributes['prot'];
      result.Port:=StrToIntDef(Attributes['port'],0);
      result.Path:=Attributes['path'];
      if (LowerCase(Attributes['verb'])='true') or (Attributes['verb']='1') then
        result.Verbosity:=True else result.Verbosity:=False;
      result.User:=Attributes['user'];
      result.Session:=Attributes['sess'];
      result.Description:=Attributes['desc'];
    end;
  except
  end;
end;

function HostCount:integer;
begin
  try
    result:=DataModule1.XML1.DocumentElement.ChildNodes['Hosts'].ChildNodes.Count;
  except
    result:=0;
  end;
end;

function ChildNodeName(i:integer): string;
begin
  try
    result:=DataModule1.XML1.DocumentElement.ChildNodes['Hosts'].ChildNodes[i].LocalName;
  except
    result:='';
  end;
end;

function DeleteXmlHost(n: string):boolean;
begin
  result:=True;
  try
    DataModule1.XML1.DocumentElement.ChildNodes['Hosts'].ChildNodes.Delete(n);
  except
    result:=False;
  end;
end;

function LoadProtocols: boolean;
var
  i: integer;
begin
  result:=True;
  try
    with DataModule1.XML1.DocumentElement.ChildNodes['Protocols'] do
    begin
      SetLength(Protocols,ChildNodes.Count);
      SetLength(ExecCommands,ChildNodes.Count);
      if ChildNodes.Count=0 then result:=False;
      for i:=0 to ChildNodes.Count-1 do
      begin
        Protocols[i].Name:=ChildNodes[i].LocalName;
        Protocols[i].Ip:=StrToIntDef(ChildNodes[i].Attributes['ip'],0);
        Protocols[i].DefIp:=ChildNodes[i].Attributes['defip'];
        Protocols[i].Verbosity:=StrToIntDef(ChildNodes[i].Attributes['verbosity'],0);
        if (ChildNodes[i].Attributes['defverbosity']='1') or (LowerCase(ChildNodes[i].Attributes['defverbosity'])='true') then
          Protocols[i].DefVerosity:=True else Protocols[i].DefVerosity:=False;
        Protocols[i].Port:=StrToIntDef(ChildNodes[i].Attributes['port'],0);
        Protocols[i].DefPort:=StrToIntDef(ChildNodes[i].Attributes['defport'],0);
        Protocols[i].User:=StrToIntDef(ChildNodes[i].Attributes['user'],0);
        Protocols[i].DefUser:=ChildNodes[i].Attributes['defuser'];
        Protocols[i].Path:=StrToIntDef(ChildNodes[i].Attributes['path'],0);
        Protocols[i].DefPath:=ChildNodes[i].Attributes['defpath'];
        Protocols[i].Session:=StrToIntDef(ChildNodes[i].Attributes['session'],0);
        Protocols[i].DefSession:=ChildNodes[i].Attributes['defsession'];
        Protocols[i].SavedSession:=ChildNodes[i].Attributes['savedsession'];
        ExecCommands[i].Protocol:=ChildNodes[i].LocalName;
        ExecCommands[i].Exec:=ChildNodes[i].Attributes['exec'];
        ExecCommands[i].Option:=ChildNodes[i].Attributes['option'];
      end;
    end;
  except
    result:=False;
  end;
end;

function LoadTypes: boolean;
var
  i: integer;
begin
  result:=True;
  try
    with DataModule1.XML1.DocumentElement.ChildNodes['Types'] do
    begin
      SetLength(DevIcons,ChildNodes.Count);
      if ChildNodes.Count=0 then result:=False;
      for i:=0 to ChildNodes.Count-1 do
      begin
        DevIcons[i].id:=StrToIntDef(copy(ChildNodes[i].LocalName,3,32),-1);
        DevIcons[i].Name:=ChildNodes[i].Attributes['name'];
        DevIcons[i].FileImage:=ChildNodes[i].Attributes['image'];
      end;
    end;
  except
    result:=False;
  end;
end;

end.

